#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#include <globus_toolkit.h>

#include <globus_nexus.h>
#include <globus_gram_myjob.h>
#include <globus_duroc_runtime.h>
#include <globus_duroc_bootstrap.h>
#include <globus_common.h>

static int myProcID, numProcs;
static int numMsgs;
static int jobSize;
static globus_nexus_startpoint_t localSP;
static globus_nexus_startpoint_t *SP_ar;

static globus_nexus_endpoint_t localEP;
static globus_monitor_t globus_Monitor;


static int ship_startpoints(globus_nexus_startpoint_t shipTo,
		     globus_nexus_startpoint_t *SP_ar);
static void startpoint_handler(globus_nexus_endpoint_t *endpoint,
			       globus_nexus_buffer_t *buffer,
			       globus_bool_t is_nonthreaded_handler);

#define STARTPOINT_HANDLER_ID 0

static globus_nexus_handler_t handler_table[] =
{
  {GLOBUS_NEXUS_HANDLER_TYPE_NON_THREADED,
   startpoint_handler},
};
#define HANDLER_TABLE_SIZE 1

int getMyID(void) { return myProcID; }
int getNumProcs(void) { return numProcs; }

int duroc_init(int argc, char **argv)
{
  int rc;
  int i;
  char text[256];

  if ((rc = globus_module_activate(GLOBUS_NEXUS_MODULE)) != GLOBUS_SUCCESS)
  {
    globus_libc_printf("Init for nexus failed, rc=0x%08x.\n", rc);
    fflush(NULL);
    return rc;
  }
  
  if ((rc = globus_module_activate(GLOBUS_GRAM_MYJOB_MODULE))
      != GLOBUS_SUCCESS)
  {
    globus_libc_printf("Init for myjob failed, rc=0x%08x.\n", rc);
    fflush(NULL);
    return rc;
  }

  if ((rc = globus_module_activate(GLOBUS_DUROC_RUNTIME_MODULE))
      != GLOBUS_SUCCESS)
  {
    globus_libc_printf("Init for duroc_runtime failed, rc=0x%08x.\n", rc);
    fflush(NULL);
    return rc;
  }

  if ((rc = globus_module_activate(GLOBUS_DUROC_BOOTSTRAP_MODULE))
      != GLOBUS_SUCCESS)
  {
    globus_libc_printf("Init for duroc_bootstrap failed, rc=0x%08x.\n", rc);
    return rc;
  }
  return GLOBUS_SUCCESS;
}

int duroc_rendezvous(globus_nexus_startpoint_t localSP,
		     globus_nexus_startpoint_t **remoteSP_ar)
{
  int i;
  int rc;
  
  globus_duroc_runtime_barrier();

  globus_duroc_runtime_intra_subjob_rank(&myProcID);
  globus_duroc_runtime_intra_subjob_size(&numProcs);

  globus_monitor_init(&globus_Monitor);

  if(numProcs==1) {
    *remoteSP_ar=(globus_nexus_startpoint_t *) globus_malloc(sizeof(globus_nexus_startpoint_t)*numProcs);
    remoteSP_ar[0]=&localSP;
    return GLOBUS_SUCCESS;
    }
  
  globus_duroc_bootstrap_master_sp_vector(&localSP, &jobSize, &SP_ar);

  /* I'm the master send startpoint array */
  if(myProcID==0) {
    for(i=1; i<numProcs; i++) {
      globus_libc_printf("%d: shipping startpoints to %d\n", myProcID, i);
      fflush(NULL);
      if(ship_startpoints(SP_ar[i], SP_ar) != GLOBUS_SUCCESS) {
	    globus_libc_printf("MASTER: Ship to %d failed\n", i);
	    fflush(NULL);
	    return rc;
      }
    }
  }      
  /* I'm not the master, wait for my startpoints */
  else {
    SP_ar=(globus_nexus_startpoint_t *) globus_malloc(sizeof(globus_nexus_startpoint_t)*numProcs);
    globus_monitor_wait(&globus_Monitor);
    globus_libc_printf("%d: got startpoints!\n", myProcID);
    fflush(NULL);
  }

  *remoteSP_ar=SP_ar;

  return GLOBUS_SUCCESS;
}

int ship_startpoints(globus_nexus_startpoint_t shipTo,
		     globus_nexus_startpoint_t *SP_ar)
{
  globus_nexus_buffer_t buffer;
  int buf_size, rc, i;
  globus_nexus_startpoint_t *copy_ar;

  copy_ar=(globus_nexus_startpoint_t *)globus_malloc(sizeof(globus_nexus_startpoint_t)*numProcs);
  for(i=0; i<numProcs; i++)
    globus_nexus_startpoint_copy(&copy_ar[i], &SP_ar[i]);
  buf_size = globus_nexus_sizeof_startpoint(copy_ar, numProcs);
  globus_nexus_buffer_init(&buffer, buf_size, 0);
  globus_nexus_put_startpoint_transfer(&buffer, copy_ar, numProcs);

  rc=globus_nexus_send_rsr(&buffer,
			   &shipTo,
			   STARTPOINT_HANDLER_ID,
			   GLOBUS_TRUE,
			   GLOBUS_FALSE);
  return rc;
}

static void startpoint_handler(globus_nexus_endpoint_t *endpoint,
			globus_nexus_buffer_t *buffer,
			globus_bool_t is_nonthreaded_handler)
{
  globus_nexus_get_startpoint(buffer, SP_ar, numProcs);

  /* free the buffer up */
  globus_nexus_buffer_destroy(buffer);
  globus_monitor_release(&globus_Monitor, GLOBUS_MONITOR_DONE);
}

int duroc_abort(void)
{
  globus_gram_myjob_kill();
  return duroc_cleanup();
}

int duroc_cleanup(void)
{
  int rc, i;

  globus_monitor_destroy(&globus_Monitor);
  /*globus_nexus_startpoint_destroy(&localSP);*/
  for(i=0; i<numProcs; i++)
	if(i!=myProcID)
      globus_nexus_startpoint_destroy(&(SP_ar[i]));
  
  
  if ((rc = globus_module_deactivate_all())
      != GLOBUS_SUCCESS)
  {
    globus_libc_fprintf(stderr,"deact failed, rc=0x%08x.\n", rc);
    fflush(NULL);
    return rc;
  }
  
  return GLOBUS_SUCCESS;
}

