#include "globus_toolkit.h"

void globus_monitor_init(globus_monitor_t *Monitor)
{
  globus_mutex_init(&Monitor->mutex, (globus_mutexattr_t *) NULL);
  globus_cond_init(&Monitor->cond, (globus_condattr_t *) NULL);
  
  /* entering the monitor and clearing the flag */
  globus_mutex_lock(&Monitor->mutex);
  Monitor->done = GLOBUS_FALSE;
  Monitor->success = GLOBUS_FALSE;
  globus_mutex_unlock(&Monitor->mutex);
}

void globus_monitor_reset(globus_monitor_t *Monitor)
{
  globus_mutex_lock(&Monitor->mutex);
  Monitor->done = GLOBUS_FALSE;
  Monitor->success = GLOBUS_FALSE;
  globus_mutex_unlock(&Monitor->mutex);
}

int globus_monitor_wait(globus_monitor_t *Monitor)
{
  int retval;
  
  globus_mutex_lock(&Monitor->mutex);
  while (!Monitor->done && !Monitor->success)
  {
    globus_cond_wait(&Monitor->cond, &Monitor->mutex);
  }
  retval=(Monitor->done==GLOBUS_TRUE) ? GLOBUS_MONITOR_DONE : GLOBUS_MONITOR_SUCCESS;
  if(Monitor->done != GLOBUS_TRUE) {
    Monitor->success=Monitor->done=GLOBUS_FALSE;
  }
  globus_mutex_unlock(&Monitor->mutex);
  return retval;
}

void globus_monitor_release(globus_monitor_t *Monitor, int trigger)
{
  globus_mutex_lock(&Monitor->mutex);
  if(trigger==GLOBUS_MONITOR_SUCCESS) {
    Monitor->success = GLOBUS_TRUE;
  }
  else {
    Monitor->done = GLOBUS_TRUE;
  }
  globus_cond_signal(&Monitor->cond);
  globus_mutex_unlock(&Monitor->mutex);
}

void globus_monitor_destroy(globus_monitor_t *Monitor)
{
  /* In case it's already locked, try and lock it first, so after the */
  /* next statement, it's always safe to unlock it before destroying */
  globus_mutex_trylock(&Monitor->mutex);
  globus_mutex_unlock(&Monitor->mutex);
  
  globus_mutex_destroy(&Monitor->mutex);
  globus_cond_destroy(&Monitor->cond);
}
