#include <stdio.h>
#include <errno.h>
#include <unistd.h>

#include "globus_nexus.h"
#include "globus_gram_myjob.h"

#include "globus_toolkit.h"

#define GS_REMOTE_HANDLER_ID 0

globus_nexus_startpoint_t *gc_remoteSP;
globus_nexus_endpoint_t *gc_localEP;

static int gc_port;
static char *gc_host;
static int gc_procid;

void gc_cleanup(void)
{
  globus_module_deactivate_all();
}  

void gc_abort(void)
{
  globus_gram_myjob_kill();
  gc_cleanup();
}


int gc_init(char **argv)
{
  int rc;
  
  rc = globus_module_activate(GLOBUS_NEXUS_MODULE);
  if(rc!=GLOBUS_SUCCESS && rc!=GLOBUS_NEXUS_ERROR_ALREADY_INITIALIZED) return NEXUS_INIT_FAILED;

  /* get the arguments and connect to creator */
  gc_procid=atoi(argv[3]);
  gc_host=argv[2];
  gc_port=atoi(argv[1]);
  return rc;
}  

int gc_connect(globus_nexus_endpoint_t *ep)
{
  int gc_attach(char *host, int port);
  int rc;
  
  gc_remoteSP=(globus_nexus_startpoint_t *)malloc(sizeof(globus_nexus_startpoint_t));
  gc_localEP=ep;
  rc=gc_attach(gc_host, gc_port);
  return rc;
}

static int pass_sp()
{
  globus_nexus_startpoint_t reply_sp;
  globus_nexus_buffer_t buffer;
  int buf_size=0, rc;
    
  globus_nexus_startpoint_bind(&reply_sp, gc_localEP);

  /*
   * Send the Startpoint back to the caller
   */
  buf_size = globus_nexus_sizeof_startpoint(&reply_sp, 1) + sizeof(int);;
  globus_nexus_buffer_init(&buffer, buf_size, 0);
  globus_nexus_put_int(&buffer, &gc_procid, 1);
  globus_nexus_put_startpoint_transfer(&buffer, (&reply_sp), 1);

  if (globus_nexus_send_rsr(&buffer,
			    gc_remoteSP,
			    GS_REMOTE_HANDLER_ID,
			    GLOBUS_TRUE,
			    GLOBUS_FALSE) != GLOBUS_SUCCESS)
  {
    return SEND_FAILED;
  }
  else return GLOBUS_SUCCESS;
}

/*
 * gc_attach()
 */
static int gc_attach(char *host, int port)
{
  char url[1024];
  int rc;
  
  sprintf(url, "x-nexus://%s:%d/", host, port);
    fflush(NULL);
  
  rc = globus_nexus_attach(url, gc_remoteSP);
  if (rc == 0) {
    rc=pass_sp();
    globus_nexus_startpoint_destroy(gc_remoteSP);
    return rc;
  }
  else {
    return CONNECT_FAILED;
  }
  
} 

