#include "HPCxx_CommCoord.h"
#include <hpcxx_rts.h>		// for hpcxx_debug_level
#include <HPCxx_Thread.h>
#include <hpcxx_rtsImpl.h>	// for hpcxx_threadID()
#include <iostream.h>

//
// This class is designed to have a separate queue for each kernel level
// thread within a context.  Since the user-level threads are non-preemptive,
// only one user-level thread at a time can access the CommCoord 
// class methods.  The bottom line: no locks are needed to protect the
// communication object queues.
//

//
// Class objects and methods (static)
//

// Create a mutex to protect communication.  This mutex may be replaced.
static HPCxx_Mutex commMutex_g;

HPCxx_Mutex* HPCxx_CommCoord::commMutex_ = &commMutex_g;


void HPCxx_CommCoord::setCommMutex(HPCxx_Mutex* mutex)
{
  commMutex_ = mutex;
}


HPCxx_Mutex* HPCxx_CommCoord::commMutex(void)
{
  return commMutex_;
}


//---- HPCxx_CommCoord --------------------------------------------------------
// Constructor.
//-----------------------------------------------------------------------------
HPCxx_CommCoord::HPCxx_CommCoord(void)
{
}


//---- handleMessage ----------------------------------------------------------
// Handles an incoming message.
//-----------------------------------------------------------------------------
void HPCxx_CommCoord::handleMessage(HPCxx_Comm& comm)
{
  int msgType = comm.messageType();

  switch (msgType) {
    case AckReplyMsg :
      hpcxx_returnHandler(comm);
      break;
    case AskForDataMsg:
      hpcxx_readHandler(comm);
      break;
    case BarrierFanInMsg:
      hpcxx_barrierFanIn(comm);
      break;
    case BarrierFanOutMsg:
      hpcxx_barrierFanOut(comm);
      break;
    case EagerPutMsg :
      hpcxx_writeHandler(comm);
      break;
    case EndOfFetchMsg:
      comm.signal();
      break;
    case RemoteFunctionMsg :
      hpcxx_invokeGFHandler(comm);
      break;
    case RemoteMethodMsg :
      hpcxx_invokeRMIHandler(comm);
      break;
    default:
      cerr << "HPCxx_CommCoord::handleMessage, unknown message type" << endl;
  }
}
