//##########################################################################
// HPC++Lib: An object-oriented class library for high performance 
//           parallel programming.
//
// Copyright (c) 1997
//	Indiana University
//	Los Alamos National Laboratory
// 
// http://www.acl.lanl.gov/Tulip
// http://www.extreme.indiana.edu/HPC++
// 
// Pete Beckman, Dennis Gannon, Todd Green, Liz Johnson
//
// HPC++_VER: $Id: hpcxx_invoke.h,v 1.1 1999/04/17 01:19:32 kate Exp $
//##########################################################################

// $Id: hpcxx_invoke.h,v 1.1 1999/04/17 01:19:32 kate Exp $

#ifndef __HPCXX_INVOKE_H_
#define __HPCXX_INVOKE_H_

//# include files
#include <HPCxx_GlobalPtr.h>


template <class C>
class HPCxx_RegisteredType {
  int key;
 public:
  HPCxx_RegisteredType() {
    C *p = NULL;
    key = hpcxx_registerClass(p);
  }
};


//template < class T >
//void hpcxx_pack(HPCxx_Buffer &buffer, HPCxx_GlobalPtr<T> *r, int count);

// invoke a remote global function that takes no args
// and returns void
//
int hpcxx_invoke(HPCxx_ContextID &gp, int funcID);

// invoke a remote global function that takes no args
// and returns a value
//
template<class R>
int hpcxx_invoke(HPCxx_ContextID &gp, R &res, int funcID);

// invoke a remote global function that takes one arg
// and returns a value
//
template<class R, class A1>
int hpcxx_invoke(HPCxx_ContextID &target, R &result, int funcID, A1 &a1);

// invoke a remote global function that takes two args
// and returns a value
//
template<class R, class A1, class A2>
int hpcxx_invoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2);

// invoke a remote global function that takes three args
// and returns a value
//
template<class R, class A1, class A2, class A3>
int hpcxx_invoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3);

// invoke a remote global function that takes four args
// and returns a value
//
template<class R, class A1, class A2, class A3, class A4>
int hpcxx_invoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3, A4 a4);

// invoke a remote global function that takes five args
// and returns a value
//
template<class R, class A1, class A2, class A3, class A4, class A5>
int hpcxx_invoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5);

// invoke a remote global function that takes six args
// and returns a value
//
template<class R, class A1, class A2, class A3, class A4, class A5, class A6>
int hpcxx_invoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6);

// invoke a remote global function that takes seven args
// and returns a value
//
template<class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
int hpcxx_invoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7);

// invoke a remote global function that takes eight args
// and returns a value
//
template<class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
int hpcxx_invoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7, A8 a8);

//
// Asynchronous invocation of remote global functions
//

// asynchronously invoke a remote global function that takes no args
// and returns void
//
int hpcxx_ainvoke(HPCxx_ContextID &gp, int funcID);

// asynchronously invoke a remote global function that takes no args
// and returns a value
//
template<class R>
int hpcxx_ainvoke(HPCxx_ContextID &gp, R &res, int funcID);

// asynchronously invoke a remote global function that takes one arg
// and returns a value
//
template<class R, class A1>
int hpcxx_ainvoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1);

// asynchronously invoke a remote global function that takes two args
// and returns a value
//
template<class R, class A1, class A2>
int hpcxx_ainvoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2);

// asynchronously invoke a remote global function that takes three args
// and returns a value
//
template<class R, class A1, class A2, class A3>
int hpcxx_ainvoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3);

// asynchronously invoke a remote global function that takes four args
// and returns a value
//
template<class R, class A1, class A2, class A3, class A4>
int hpcxx_ainvoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3, A4 a4);

// asynchronously invoke a remote global function that takes five args
// and returns a value
//
template<class R, class A1, class A2, class A3, class A4, class A5>
int hpcxx_ainvoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5);

// asynchronously invoke a remote global function that takes six args
// and returns a value
//
template<class R, class A1, class A2, class A3, class A4, class A5, class A6>
int hpcxx_ainvoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6);

// asynchronously invoke a remote global function that takes seven args
// and returns a value
//
template<class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
int hpcxx_ainvoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7);

// asynchronously invoke a remote global function that takes eight args
// and returns a value
//
template<class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
int hpcxx_ainvoke(HPCxx_ContextID &gp, R &res, int funcID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7, A8 a8);

//
//
// Remote member function invocation
//
//

// invoke a remote member function that takes no args
// and returns void
//
template<class P>
int  hpcxx_minovke(HPCxx_GlobalPtr<P> *gp, int mFuncID);

// invoke a remote member function that takes no args
// and returns a value
//
template<class P, class R>
int  hpcxx_minovke(HPCxx_GlobalPtr<P> *gp, R &res, int mFuncID);

// invoke a remote member function that takes one arg
// and returns a value
//
template<class P, class R, class A1>
int  hpcxx_minovke(HPCxx_GlobalPtr<P> *gp, R &res, int mFuncID, A1 a1);

// invoke a remote member function that takes two args
// and returns a value
//
template<class P, class R, class A1, class A2>
int  hpcxx_minovke(HPCxx_GlobalPtr<P> *gp, R &res, int mFuncID, A1 a1, A2 a2);

// invoke a remote member function that takes three args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3>
int  hpcxx_minovke(HPCxx_GlobalPtr<P> *gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3);

// invoke a remote member function that takes four args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3, class A4>
int  hpcxx_minovke(HPCxx_GlobalPtr<P> *gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3, A4 a4);

// invoke a remote member function that takes five args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3, class A4, class A5>
int  hpcxx_minovke(HPCxx_GlobalPtr<P> *gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5);

// invoke a remote member function that takes six args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3, class A4, class A5, class A6>
int  hpcxx_minovke(HPCxx_GlobalPtr<P> *gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6);

// invoke a remote member function that takes seven args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
int  hpcxx_minovke(HPCxx_GlobalPtr<P> *gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7);

// invoke a remote member function that takes eight args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
int  hpcxx_minovke(HPCxx_GlobalPtr<P> *gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7, A8 a8);


//
// Asynchronous invocation of remote member functions
//

// asynchronously invoke a remote member function that takes no args
// and returns void
//
template<class P>
int hpcxx_ainvoke(HPCxx_GlobalPtr<P> &gp, int mFuncID);

// asynchronously invoke a remote member function that takes no args
// and returns a value
//
template<class P, class R>
int hpcxx_ainvoke(HPCxx_GlobalPtr<P> &gp, R &res, int mFuncID);

// asynchronously invoke a remote member function that takes one arg
// and returns a value
//
template<class P, class R, class A1>
int hpcxx_ainvoke(HPCxx_GlobalPtr<P> &gp, R &res, int mFuncID, A1 a1);

// asynchronously invoke a remote member function that takes two args
// and returns a value
//
template<class P, class R, class A1, class A2>
int hpcxx_ainvoke(HPCxx_GlobalPtr<P> &gp, R &res, int mFuncID, A1 a1, A2 a2);

// asynchronously invoke a remote member function that takes three args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3>
int hpcxx_ainvoke(HPCxx_GlobalPtr<P> &gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3);

// asynchronously invoke a remote member function that takes four args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3, class A4>
int hpcxx_ainvoke(HPCxx_GlobalPtr<P> &gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3, A4 a4);

// asynchronously invoke a remote member function that takes five args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3, class A4, class A5>
int hpcxx_ainvoke(HPCxx_GlobalPtr<P> &gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5);

// asynchronously invoke a remote member function that takes six args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3, class A4, class A5, class A6>
int hpcxx_ainvoke(HPCxx_GlobalPtr<P> &gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6);

// asynchronously invoke a remote member function that takes seven args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
int hpcxx_ainvoke(HPCxx_GlobalPtr<P> &gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7);

// asynchronously invoke a remote member function that takes eight args
// and returns a value
//
template<class P, class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
int hpcxx_ainvoke(HPCxx_GlobalPtr<P> &gp, R &res, int mFuncID, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7, A8 a8);

#endif
