//##########################################################################
// HPC++Lib: An object-oriented class library for high performance 
//           parallel programming.
//
// Copyright (c) 1997
//	Indiana University
//	Los Alamos National Laboratory
// 
// http://www.acl.lanl.gov/Tulip
// http://www.extreme.indiana.edu/HPC++
// 
// Pete Beckman, Dennis Gannon, Todd Green, Liz Johnson
//
// HPC++_VER: $Id: hpcxx_register.h,v 1.1 1999/04/17 01:19:32 kate Exp $
//##########################################################################

#ifndef __HPCXX_REGISTER_H_
#define __HPCXX_REGISTER_H_

//# include files
#include <hpcxx_types.h>

// Allocate an identifier space.  Two spaces are allocated and
// reserved by HPC++ Lib:
//     HPCXX_REGISTER_IDSPACE
//     HPCXX_GROUP_IDSPACE
// Return the next available idspace or HPCXX_INVALID_IDSPACE
// if no spaces are left
//
int hpcxx_createIDSpace();

// After having allocated an id space, id's may be obtained
// via the two functions.
//
// hpcxx_getGlobalID implies a global communication and guarantee's
// synchronization across contexts
//
// hpcxx_getLocalID implies no communication, but it is the user's
// responsibility to insure that keys are synchronized across contexts
// (SPMD mode)
hpcxx_id_t hpcxx_getGlobalID(int idSpace);
hpcxx_id_t hpcxx_getLocalID(int idSpace);

// register a class
template<class C>
hpcxx_id_t hpcxx_registerClass(C *c); 


// register a global function that takes no arguments
template<class R>
hpcxx_id_t hpcxx_register(R (*f) (void), hpcxx_id_t registerID );

// Register a global function that takes one argument
template<class R, class A1>
hpcxx_id_t hpcxx_register(R (*f) (A1), hpcxx_id_t registerID );

// Register a global function that takes two arguments
template<class R, class A1, class A2>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2), hpcxx_id_t registerID );

// Register a global function that takes three arguments
template<class R, class A1, class A2, class A3>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3), hpcxx_id_t registerID );

// Register a global function that takes four arguments
template<class R, class A1, class A2, class A3, class A4>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3, A4), hpcxx_id_t registerID );

// Register a global function that takes five arguments
template<class R, class A1, class A2, class A3, class A4, class A5>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3, A4, A5), hpcxx_id_t registerID );

// Register a global function that takes six arguments
template<class R, class A1, class A2, class A3, class A4, class A5, class A6>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3, A4, A5, A6), 
              hpcxx_id_t registerID );

// Register a global function that takes seven arguments
template<class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3, A4, A5, A6, A7), hpcxx_id_t registerID );

// Register a global function that takes eight arguments
template<class R, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
hpcxx_id_t hpcxx_register(R (*f) (A1, A2, A3, A4, A5, A6, A7, A8), hpcxx_id_t registerID );

// Register a member function that takes no arguments
template<class R, class C>
hpcxx_id_t hpcxx_register(R (C::*mf) (void), hpcxx_id_t registerID );

// Register a member function that takes one argument
template<class R, class C, class A1>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1), hpcxx_id_t registerID );

// Register a member function that takes two arguments
template<class R, class C, class A1, class A2>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2), hpcxx_id_t registerID );

// Register a member function that takes three arguments
template<class R, class C, class A1, class A2, class A3>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3), hpcxx_id_t registerID );

// Register a member function that takes four arguments
template<class R, class C, class A1, class A2, class A3, class A4>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3, A4), hpcxx_id_t registerID );

// Register a member function that takes five arguments
template<class R, class C, class A1, class A2, class A3, class A4, class A5>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3, A4, A5), hpcxx_id_t registerID );

// Register a member function that takes six arguments
template<class R, class C, class A1, class A2, class A3, class A4, class A5, class A6>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3, A4, A5, A6), hpcxx_id_t registerID );

// Register a member function that takes seven arguments
template<class R, class C, class A1, class A2, class A3, class A4, class A5, class A6, class A7>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3, A4, A5, A6, A7), hpcxx_id_t registerID );

// Register a member function that takes eight arguments
template<class R, class C, class A1, class A2, class A3, class A4, class A5, class A6, class A7, class A8>
hpcxx_id_t hpcxx_register(R (C::*mf) (A1, A2, A3, A4, A5, A6, A7, A8), hpcxx_id_t registerID );

#endif
